<?php
namespace Zeedhi\DevOps\Model\Devops\Repositories;

use Zeedhi\DevOps\Model\Traits\TQuery;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class ProductVersion extends \Doctrine\ORM\EntityRepository {

    use TQuery;

    public function findProductVersionWithConcatVersionByProduct($productId,$organizationId, $limit = 500, $offset = 1, $filterString = '') {

        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findLtsVersionWithQuantityFilter($productId, $quantity, $organizationId) {
        $query = <<<SQL
            SELECT *
              FROM (SELECT A.*, ROWNUM AS ROW_NUMBER
                      FROM (SELECT PR.PRODUCT_ID, PR.ID, PR.VERSION, PR.REVISION
                              FROM PRODUCT_VERSION PR
                             WHERE PR.PRODUCT_ID = :PRODUCT
                               AND PR.ORGANIZATION_ID = :ORGANIZATION
                               AND PR.IS_LTS = :IS_LTS
                          ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC) A)
            WHERE ROW_NUMBER > :QUANTITY
SQL;
        $params = array(
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId,
            'QUANTITY'     => $quantity,
            'IS_LTS'       => \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion::IS_LTS_YES,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductOrgByDevopsProductId($devopsId, $nrorg) {
        $query = <<<SQL
		SELECT OMO.ID 
          FROM OM_PRODUTO_ORG OMO
          JOIN OM_PRODUTO OPR ON OPR.ID = OMO.NRPRODUTO_ID
         WHERE OPR.NREXTERNAL_PRODUCT_ID = :DEVOPS_ID 
           AND OMO.NRORG_ID = :NRORG_ID
SQL;
        $param = array( 
            'DEVOPS_ID' => $devopsId, 
            'NRORG_ID'  => $nrorg 
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findProductAvailableVersions($nrProdutoOrgId, $nrProdutoOrgParentId = null) {
        $parentFilter = "";
        if($nrProdutoOrgParentId) {
            $parentFilter = " AND OV.PARENT_NRPROD_ORG_ID = $nrProdutoOrgParentId";
        }
        $query = <<<SQL
		  SELECT PV.ID
            FROM OM_PRODUTO_ORG_VERSAO OV
            JOIN PRODUCT_VERSION PV ON PV.ID = OV.PRODUCT_VERSION_ID
           WHERE OV.NRPRODUTO_ORG_ID = :NRPRODUTO_ORG_ID
                 $parentFilter
        ORDER BY PV.VERSION DESC, PV.REVISION DESC

SQL;
        $param = array( 'NRPRODUTO_ORG_ID' => $nrProdutoOrgId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findParentProductAvailableVersions($nrProdutoOrgId){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT OV.PARENT_NRPROD_ORG_ID 
                   FROM OM_PRODUTO_ORG_VERSAO OV
                  WHERE OV.NRPRODUTO_ORG_ID = :NRPRODUTO_ORG_ID )

SQL;
        $param = array( 'NRPRODUTO_ORG_ID' => $nrProdutoOrgId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    
        return $registers;
    }

    public function findOmProdutoByDevopsProductId($devopsId){
        $query = <<<SQL
         SELECT * FROM OM_PRODUTO WHERE NREXTERNAL_PRODUCT_ID = :NREXTERNAL_PRODUCT_ID
SQL;
        $param = array( 'NREXTERNAL_PRODUCT_ID' => $devopsId);
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findMaxProductVersionWithFilter($productId, $filter) {
        $query = <<<SQL
			SELECT *
              FROM (SELECT PR.ID AS PRODUCT_VERSION_ID, (PR.VERSION || '.' || PR.REVISION  || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                           $filter
                     ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
             WHERE ROWNUM = 1
SQL;

        $params = array( 'PRODUCT' => $productId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMaxProductVersionWithFilterDeploy($productId, $filter) {
        $query = <<<SQL
		  SELECT PR.ID AS PRODUCT_VERSION_ID, (PR.VERSION || '.' || PR.REVISION  || '.' || PR.BUILD) AS VERSION
            FROM PRODUCT_VERSION PR
            JOIN (SELECT *
                    FROM (SELECT PR.ID, PR.VERSION, PR.REVISION
                            FROM PRODUCT_VERSION PR
                           WHERE PR.PRODUCT_ID = :PRODUCT
                             AND PR.IS_REVISION = 'Y'
                             AND PR.IS_LTS = 'Y'
                                 $filter
                        ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
                   WHERE ROWNUM = 1
                 ) PL ON PL.VERSION  = PR.VERSION
                     AND PL.REVISION = PR.REVISION
           WHERE PR.PRODUCT_ID = :PRODUCT
        ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC
SQL;

        $params = array( 'PRODUCT' => $productId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProdcutVersionByProject($project)
    {
        $query = <<<SQL
           SELECT PV.VERSION , PV.REVISION FROM PROJECT P
             LEFT JOIN PRODUCT_VERSION PV ON P.PRODUCT_VERSION_ID = PV.ID
           WHERE P.ID = :PROJECT

SQL;
        $params = array(
            'PROJECT'      => $project
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findProductVersionConcatenated($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '')
    {

        $query = <<<SQL
            SELECT *
              FROM (SELECT NULL AS PRODUCT_ID, NULL AS ID, 'Última versão' AS VERSION
                      FROM DUAL)
             WHERE 1=1
             $filterString
         UNION ALL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
             WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductVersionLtsConcatenatedWithoutBuild($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '', $filterNotLTS = 0)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION, 
                           DECODE(PR.IS_LTS, 'Y', 'LTS', 'Não LTS') AS IS_LTS
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND (PR.IS_LTS = 'Y' OR PR.ID IN ($filterNotLTS))
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findProductVersionLtsConcatenatedWithoutBuildNext($productId, $organizationId, $filterNotLTS = 0)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT NULL AS PRODUCT_ID, 'Última versão' AS ID, 'Última versão' AS VERSION, NULL AS IS_LTS
                      FROM DUAL)
             WHERE 1=1
         UNION ALL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, TO_CHAR(PR.ID), (PR.VERSION || '.' || PR.REVISION) AS VERSION, 
                           DECODE(PR.IS_LTS, 'Y', 'LTS', 'Não LTS') AS IS_LTS
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND (PR.IS_LTS = 'Y' OR PR.ID IN ($filterNotLTS))
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query, $params);
    }

    public function findLtsCurrentProductVersionFromVersion($productId, $organizationId, $version, $revision, $productVersionId = false)
    {
        $productVersionIdFilter = "";
        if($productVersionId) {
            $productVersionIdFilter = "AND PR.ID <> $productVersionId";
        }
        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                       AND (PR.VERSION > :VERSION 
                            OR (PR.VERSION = :VERSION AND PR.REVISION >= :REVISION)
                           )
                       $productVersionIdFilter
                  ORDER BY PR.VERSION ASC, PR.REVISION ASC)
            WHERE ROWNUM = 1
SQL;
        $params = array(
            'VERSION'      => $version,
            'REVISION'     => $revision,
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    public function findLTSProductVersionToRemoveByVersion($product, $version)
    {   
        $query = <<<SQL
            SELECT ID, (VERSION || '.' || REVISION) as VERSION, TO_CHAR(CREATED, 'DD/MM/YYYY') AS CREATION_DATE
              FROM PRODUCT_VERSION PV
             WHERE IS_LTS = 'Y'
               AND PRODUCT_ID = :PRODUCT_ID
               AND TO_NUMBER(LPAD(VERSION, 5, '0') || LPAD(REVISION, 5, '0')) > TO_NUMBER(LPAD(:VERSION, 5, '0') || LPAD(:REVISION, 5, '0'))
             ORDER BY VERSION DESC, REVISION DESC

SQL;
        $params = array(
            'VERSION'      => $version['VERSION'],
            'REVISION'      => $version['REVISION'],
            'PRODUCT_ID'     => $product,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findNextVersionsLts($productId, $organizationId, $version, $revision, $productVersionId)
    {
        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                       AND (PR.VERSION > :VERSION 
                            OR (PR.VERSION = :VERSION AND PR.REVISION >= :REVISION)
                           )
                       AND PR.ID <> :PRODUCT_VERSION_ID
                  ORDER BY PR.VERSION ASC, PR.REVISION ASC)
SQL;

        $params = array(
            'VERSION'            => $version,
            'REVISION'           => $revision,
            'PRODUCT'            => $productId,
            'ORGANIZATION'       => $organizationId,
            'PRODUCT_VERSION_ID' => $productVersionId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findLtsVersionWithoutBuildWithZeroVersion($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT NULL AS PRODUCT_ID, NULL AS ID, '0.0' AS VERSION
                      FROM DUAL)
             WHERE 1=1
             $filterString
         UNION ALL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCurrentBuildFromVersion($productId, $organizationId, $version, $revision)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID,
                           PR.VERSION AS PR_VERSION, PR.REVISION AS PR_REVISION, PR.BUILD AS PR_BUILD,
                           (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.VERSION = :VERSION
                       AND PR.REVISION = :REVISION
                  ORDER BY PR.BUILD DESC)
             WHERE ROWNUM = 1
SQL;
        $params = array(
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId,
            'VERSION'      => $version,
            'REVISION'     => $revision
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCurrentRevisionVersion($productId)
    {
        $query = <<<SQL
            SELECT ID, VERSION, REVISION, PRODUCT_ID
              FROM PRODUCT_VERSION
             WHERE PRODUCT_ID = :PRODUCT_ID
               AND IS_REVISION = 'Y'
             ORDER BY VERSION DESC, REVISION DESC
             FETCH FIRST 1 ROW ONLY
SQL;
        $params = array (
            'PRODUCT_ID'   => $productId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findLTSProductVersionToRemove($productId, $filterDate)
    {
        $query = <<<SQL
            SELECT ID, (VERSION || '.' || REVISION) AS VERSION, TO_CHAR(CREATED, 'DD/MM/YYYY') AS CREATION_DATE
              FROM PRODUCT_VERSION PV
             WHERE IS_LTS = 'Y'
               AND PRODUCT_ID = :PRODUCT_ID
               AND TRUNC(CREATED) <= TO_DATE(:FILTER_DATE, 'DD/MM/YYYY')
             ORDER BY VERSION DESC, REVISION DESC
SQL;
        $params = array (
            'PRODUCT_ID'  => $productId,
            'FILTER_DATE' => $filterDate,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

     public function versionWithRelationshipWithHomologatedSubRelease($productVersionIds)
    {
        $query = <<<SQL
            SELECT PV.ID, (PV.VERSION || '.' || PV.REVISION) AS VERSION
              FROM PRODUCT_VERSION  PV
              JOIN SUBRELEASE SR ON SR.PRODUCT_VERSION_ID = PV.ID 
                                AND SR.STATUS <> 'H'
                                AND SR.IS_ACTIVE = 'Y'
             WHERE PV.ID IN ($productVersionIds)
          GROUP BY PV.ID, PV.VERSION, PV.REVISION
SQL;

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
}
